@@ -4,8 +4,8 @@
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
  * In applying this licence, ECMWF does not waive the privileges and immunities
- * granted to it by virtue of its status as an intergovernmental organisation nor
- * does it submit to any jurisdiction.
+ * granted to it by virtue of its status as an intergovernmental organisation
+ * nor does it submit to any jurisdiction.
  */
 
 /*! \file Proj4Projection.cc
@@ -25,8 +25,12 @@
 #include <Polyline.h>
 #include <Proj4Projection.h>
 #include <Text.h>
+
+#include <cmath>
+
 #include "MagConfig.h"
 #include "MetaData.h"
+#include "Value.h"
 
 namespace magics {
 
@@ -44,21 +48,24 @@ public:
         initMethods_["tpers"]       = &Epsg::tpersinit;
         initMethods_["polar_north"] = &Epsg::polarinit;
         initMethods_["polar_south"] = &Epsg::polarsouthinit;
+        initMethods_["EPSG:32661"]  = &Epsg::epsg32661;
+        initMethods_["EPSG:32761"]  = &Epsg::epsg32761;
+        
     }
     string name_;
     string definition_;
 
     typedef void (Epsg::*InitMethod)(const Proj4Projection&);
-    typedef void (Epsg::*Method)(const json_spirit::Value&);
+    typedef void (Epsg::*Method)(const Value&);
     map<string, Method> methods_;
     map<string, InitMethod> initMethods_;
 
-    void definition(const json_spirit::Value& value) { definition_ = value.get_value<string>(); }
-    void minlon(const json_spirit::Value& value) { minlon_ = value.get_value<double>(); }
-    void minlat(const json_spirit::Value& value) { minlat_ = value.get_value<double>(); }
-    void maxlon(const json_spirit::Value& value) { maxlon_ = value.get_value<double>(); }
-    void maxlat(const json_spirit::Value& value) { maxlat_ = value.get_value<double>(); }
-    void method(const json_spirit::Value& value) { method_ = value.get_value<string>(); }
+    void definition(const Value& value) { definition_ = value.get_value<string>(); }
+    void minlon(const Value& value) { minlon_ = value.get_value<double>(); }
+    void minlat(const Value& value) { minlat_ = value.get_value<double>(); }
+    void maxlon(const Value& value) { maxlon_ = value.get_value<double>(); }
+    void maxlat(const Value& value) { maxlat_ = value.get_value<double>(); }
+    void method(const Value& value) { method_ = value.get_value<string>(); }
 
     void init(const Proj4Projection& from) {
         map<string, InitMethod>::iterator initmethod = initMethods_.find(name_);
@@ -76,8 +83,8 @@ public:
         ostringstream def;
         def << "+proj=geos +h=42164000 +ellps=WGS84 +lon_0=" << from.vertical_longitude_;
 
-        // def << "++proj=tpers    +ellps=WGS84 +h=5000000  +lat_0=20 +lon_0=-60 +x_0=0 +y_0=0 +azi=20 +tilt=0
-        // +units=m";
+        // def << "++proj=tpers    +ellps=WGS84 +h=5000000  +lat_0=20 +lon_0=-60
+        // +x_0=0 +y_0=0 +azi=20 +tilt=0 +units=m";
         definition_ = def.str();
     }
 
@@ -85,29 +92,54 @@ public:
         ostringstream def;
 
         def << "+proj=tpers +ellps=WGS84 +h=" << from.projection_height_;
-        def << "  +lat_0=" << from.projection_view_latitude_;
+        def << " +lat_0=" << from.projection_view_latitude_;
         def << " +lon_0=" << from.projection_view_longitude_;
         def << " +x_0=0 +y_0=0 +azi=" << from.projection_azimuth_;
         def << " +tilt=" << from.projection_tilt_ << "  +units=m";
+
+
         definition_ = def.str();
+        // definition_ = "+proj=tpers +h=5500000 +lat_0=40";
     }
 
     void polarinit(const Proj4Projection& from) {
         ostringstream def;
 
-        def << "+proj=stere +lat_0=90 +lat_ts=60 +lon_0=" << from.vertical_longitude_;
-        def << " +k=0.994 +x_0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs";
+        def << "+proj=stere +lat_0=90 ";
+        def << " +lon_0=" << from.vertical_longitude_;
+        def << " +k=0.994 +x_0=2000000 +y_0=2000000  "
+               "+units=m";
         definition_ = def.str();
     }
 
     void polarsouthinit(const Proj4Projection& from) {
         ostringstream def;
 
-        def << "+proj=stere +lat_0=-90 +lat_ts=-60 +lon_0=" << from.vertical_longitude_;
-        def << " +k=0.994 +x_0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 +units=m +no_defs";
+        def << "+proj=stere +lat_0=-90 +lat_ts=" << from.true_scale_south_;
+        def << " +lon_0=" << from.vertical_longitude_;
+        def << "  +x_0=2000000 +y_0=2000000 +ellps=WGS84 +datum=WGS84 "
+               "+units=m ";
         definition_ = def.str();
     }
 
+      void epsg32661(const Proj4Projection& from) {
+        ostringstream def;
+
+        def << "+proj=stere +lat_0=90 +lat_ts=90 ";
+        def << " +lon_0=" << from.vertical_longitude_;
+        def << " +k=0.994 +x_0=2000000 +y_0=2000000  "
+               " +datum=WGS84 +units=m";
+        definition_ = def.str();
+    }
+
+    void epsg32761(const Proj4Projection& from) {
+        ostringstream def;
+        def << "+proj=stere +lat_0=-90 +lat_ts=-90 ";
+        def << " +lon_0=" << from.vertical_longitude_;
+        def << " +k=0.994 +x_0=2000000 +y_0=2000000  "
+               " +datum=WGS84 +units=m";
+        definition_ = def.str();
+    }
 
     double minlon_;
     double minlat_;
@@ -118,7 +150,7 @@ public:
 
     static map<string, Epsg*> epsgs_;
 
-    void set(const json_spirit::Value&);
+    void set(const Value&);
 
     static Epsg* find(const Proj4Projection& from) {
         string name                       = lowerCase(from.name());
@@ -135,13 +167,12 @@ public:
     const char* definition() { return definition_.c_str(); }
 };
 
-
 class EpsgConfig : public MagConfig {
 public:
     EpsgConfig() {}
     ~EpsgConfig() {}
 
-    void callback(const string&, const json_spirit::Value&);
+    void callback(const string&, const Value&);
     void init();
     Epsg* epsg_;
 };
@@ -152,28 +183,26 @@ map<string, Epsg*> Epsg::epsgs_;
 
 void EpsgConfig::init() {
     // methods_["epsg"] =  &EpsgConfig::epsg;
-    MagConfigHandler(getEnvVariable("MAGPLUS_HOME") + MAGPLUS_PATH_TO_SHARE_ + "/epsg.json", *this);
+    MagConfigHandler(buildConfigPath("epsg.json"), *this);
 }
-void Epsg::set(const json_spirit::Value& value) {
-    ASSERT(value.type() == json_spirit::obj_type);
-    json_spirit::Object object = value.get_value<json_spirit::Object>();
-    for (vector<json_spirit::Pair>::const_iterator entry = object.begin(); entry != object.end(); ++entry) {
-        map<string, Method>::iterator method = methods_.find(entry->name_);
+void Epsg::set(const Value& value) {
+    ValueMap object = value.get_value<ValueMap>();
+    for (auto entry = object.begin(); entry != object.end(); ++entry) {
+        map<string, Method>::iterator method = methods_.find(entry->first);
         if (method != methods_.end()) {
-            ((this->*method->second)(entry->value_));
+            ((this->*method->second)(entry->second));
         }
     }
 }
-void EpsgConfig::callback(const string& name, const json_spirit::Value& value) {
+void EpsgConfig::callback(const string& name, const Value& value) {
     // here we get an Array of epsg!
 
-    ASSERT(value.type() == json_spirit::array_type);
-    json_spirit::Array values = value.get_value<json_spirit::Array>();
+    ValueList values = value.get_value<ValueList>();
     for (unsigned int i = 0; i < values.size(); i++) {
-        json_spirit::Object object = values[i].get_value<json_spirit::Object>();
-        for (vector<json_spirit::Pair>::const_iterator entry = object.begin(); entry != object.end(); ++entry) {
-            Epsg* epsg = new Epsg(entry->name_);
-            epsg->set(entry->value_);
+        ValueMap object = values[i].get_value<ValueMap>();
+        for (auto entry = object.begin(); entry != object.end(); ++entry) {
+            Epsg* epsg = new Epsg(entry->first);
+            epsg->set(entry->second);
         }
     }
 }
@@ -187,7 +216,8 @@ Proj4Projection::Proj4Projection(const string& definition) :
     gridMinLat_(DBL_MAX),
     gridMaxLon_(-DBL_MAX),
     gridMaxLat_(-DBL_MAX),
-    wraparound_(false) {
+    wraparound_(false),
+    helper_(0) {
     // init();
     EpsgConfig config;
     config.init();
@@ -198,12 +228,20 @@ Proj4Projection::Proj4Projection() :
     gridMinLat_(DBL_MAX),
     gridMaxLon_(-DBL_MAX),
     gridMaxLat_(-DBL_MAX),
-    wraparound_(false) {
+    wraparound_(false),
+    helper_(0) {
     // init();
     EpsgConfig config;
     config.init();
 }
-
+void Proj4Projection::populate(double lon, double lat, double val, vector<UserPoint>& out) const {
+    if (in(lon, lat))
+        out.push_back(UserPoint(lon, lat, val));
+    if (in(lon - 360, lat))
+        out.push_back(UserPoint(lon - 360., lat, val));
+    if (in(lon + 360, lat))
+        out.push_back(UserPoint(lon + 360., lat, val));
+}
 
 /*!
   \brief Destructor
@@ -227,19 +265,17 @@ magics::Polyline& Proj4Projection::getUserBoundingBox() const {
 void Proj4Projection::init() {
     MagLog::dev() << "Proj4Projection::init()" << *this << endl;
 
-    from_       = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84");
     projection_ = Epsg::find(*this);
-    to_         = pj_init_plus(projection_->definition());
+    helper_     = new LatLonProjP(projection_->definition());
 
-    if (!to_) {
-        MagLog::error() << pj_strerrno(pj_errno) << endl;
+    if (!helper_->valid()) {
+        // MagLog::error() << pj_strerrno(pj_errno) << endl;
         MagLog::error() << " proj4 error " << projection_->definition() << endl;
         ASSERT(false);
     }
 
-
     methods_["geos"]   = &Proj4Projection::geos;
-    methods_["tpers"]  = &Proj4Projection::geos;
+    methods_["tpers"]  = &Proj4Projection::tpers;
     methods_["polar"]  = &Proj4Projection::conic;
     methods_["conic"]  = &Proj4Projection::conic;
     methods_["simple"] = &Proj4Projection::simple;
@@ -323,7 +359,6 @@ void Proj4Projection::full() {
     }
 }
 
-
 void Proj4Projection::wrap(double& x, double& y) {}
 
 void Proj4Projection::corners() {
@@ -336,43 +371,40 @@ void Proj4Projection::corners() {
     fast_reproject(min_pcx_, min_pcy_);
     fast_reproject(max_pcx_, max_pcy_);
 
-    magics::Polyline box;
-    box.box(PaperPoint(min_pcx_, min_pcy_), PaperPoint(max_pcx_, max_pcy_));
+    delete PCEnveloppe_;
 
-    vector<magics::Polyline*> newbox;
-    PCEnveloppe_->intersect(box, newbox);
-    if (newbox.empty()) {
-        MagLog::warning() << "Proj4 : the sub-area is not valid : use global view instead" << endl;
-    }
-    else {
-        PCEnveloppe_ = newbox.front();
-    }
-    // Clear newbox
+    PCEnveloppe_ = new magics::Polyline();
+    PCEnveloppe_->box(PaperPoint(min_pcx_, min_pcy_), PaperPoint(max_pcx_, max_pcy_));
 }
 
 void Proj4Projection::centre() {}
 
 PaperPoint Proj4Projection::operator()(const UserPoint& point) const {
-    if (!from_) {
-        from_       = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84");
+    if (!helper_) {
         projection_ = Epsg::find(*this);
-        to_         = pj_init_plus(projection_->definition());
+        helper_     = new LatLonProjP(projection_->definition());
+
+        if (!helper_->valid()) {
+            // MagLog::error() << pj_strerrno(pj_errno) << endl;
+            MagLog::error() << " proj4 error " << projection_->definition() << endl;
+            ASSERT(false);
+        }
     }
     double x = point.x();
     double y = point.y();
 
-    x *= DEG_TO_RAD;
-    y *= DEG_TO_RAD;
-
-    int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+    int error = helper_->convert(x, y);
     if (error) {
-        MagLog::debug() << pj_strerrno(error) << " for " << point << endl;
-        //		NON Valeurs trop faibles en EPSG:3857 par exemple !  --> Transformation::in(pp) == True !
-        //		-> Avec ces valeurs, si grille globale, les barbules des poles sont tracées dans l'atlantique sud
-        //		en formant une couronne centrée vers( 9°W, 66°35S)
-        //		return PaperPoint(-1000000, -10000000);   .// En plus il manque 1 zero !
-        return PaperPoint(-1e10,
-                          -1e10);  // Devrait suffire pour que Transformation::in(pp) == False qqsoient from_ et to_
+        MagLog::debug() << helper_->error(error) << " for " << point << endl;
+        //		NON Valeurs trop faibles en EPSG:3857 par exemple !  -->
+        // Transformation::in(pp) == True !
+        //		-> Avec ces valeurs, si grille globale, les barbules des poles
+        // sont tracées dans l'atlantique sud 		en formant une couronne
+        // centrée vers( 9°W, 66°35S) 		return PaperPoint(1000000,
+        // 10000000);   .// En plus il manque 1 zero !
+        return PaperPoint(HUGE_VAL,
+                          HUGE_VAL);  // Devrait suffire pour que Transformation::in(pp) == False
+                                      // qqsoient from_ et to_
     }
 
     return PaperPoint(x, y, point.value_, point.missing(), point.border(), 0, point.name());
@@ -408,30 +440,25 @@ void Proj4Projection::revert(const PaperPoint& xy, UserPoint& point) const {
         first = false;
     }
     if (PCEnveloppe_->within(xy) == false) {
-        point = UserPoint(-1000, -1000);
+        point = UserPoint(HUGE_VAL, HUGE_VAL);
 
         return;
     }
 
-
     double x = xy.x();
     double y = xy.y();
 
-    int error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
+    int error = helper_->revert(x, y);
 
     if (error) {
-        MagLog::debug() << pj_strerrno(error) << endl;
-        point = UserPoint(-1000, -1000);
+        MagLog::debug() << helper_->error(error) << endl;
+        point = UserPoint(HUGE_VAL, HUGE_VAL);
         return;
     }
 
-    x *= RAD_TO_DEG;
-    y *= RAD_TO_DEG;
-
     point = UserPoint(x, y);
 }
 
-
 bool Proj4Projection::needShiftedCoastlines() const {
     // Will need w new parameter to know!
     return false;
@@ -442,11 +469,10 @@ void Proj4Projection::aspectRatio(double& width, double& height) {
     Transformation::aspectRatio(width, height);
 }
 
-
 void Proj4Projection::add(double lon, double lat) {
-    double x  = lon * DEG_TO_RAD;
-    double y  = DEG_TO_RAD * lat;
-    int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+    double x  = lon;
+    double y  = lat;
+    int error = helper_->convert(x, y);
     userEnveloppe_->push_back(PaperPoint(lon, lat));
     PCEnveloppe_->push_back(PaperPoint(x, y));
 
@@ -458,6 +484,7 @@ void Proj4Projection::add(double lon, double lat) {
         max_pcx_ = x;
     if (y > max_pcy_)
         max_pcy_ = y;
+
     if (lon < gridMinLon_)
         gridMinLon_ = lon;
     if (lat < gridMinLat_)
@@ -484,7 +511,6 @@ void Proj4Projection::conic() {
     add(projection_->maxlon_, projection_->maxlat_);
 }
 
-
 void Proj4Projection::simple() {
     userEnveloppe_->clear();
     PCEnveloppe_->clear();
@@ -517,19 +543,33 @@ void Proj4Projection::projectionSimple() {
     min_pcy_ = min_latitude_;
     max_pcx_ = max_longitude_;
     max_pcy_ = max_latitude_;
-    pj_transform(to_, from_, 1, 1, &min_longitude_, &min_latitude_, NULL);
-    pj_transform(to_, from_, 1, 1, &max_longitude_, &max_latitude_, NULL);
 
-    min_longitude_ *= RAD_TO_DEG;
-    min_latitude_ *= RAD_TO_DEG;
-    max_longitude_ *= RAD_TO_DEG;
+    cout << definition_ << endl;
+
+    
+    helper_->revert(min_longitude_, min_latitude_);
+    helper_->revert(max_longitude_, max_latitude_);
+    double x = max_longitude_;
+    double y = max_latitude_;
+
+    helper_->convert(x, y);
+
+
+
+    // cout << "[" << min_pcx_ << " " << min_pcy_ << "]-->[" << max_pcx_ << " " << max_pcy_ << "]" << endl;
+    // cout << "[" << min_longitude_ << " " << min_latitude_ << "]-->[" << max_longitude_ << " " << max_latitude_ << "]" << endl;
+    // cout << "[" << max_longitude_ << " " << max_latitude_ << "]-->[" << x << " " << y << "]" << endl;
+
+
     if (max_longitude_ < 0) {
         max_longitude_ += 360.;
     }
+    
 
-    max_latitude_ *= RAD_TO_DEG;
+   
 
 
+    
     magics::Polyline box;
     box.box(PaperPoint(min_pcx_, min_pcy_), PaperPoint(max_pcx_, max_pcy_));
 
@@ -546,7 +586,6 @@ void Proj4Projection::projectionSimple() {
     coordinates_system_ = "latlon";
 }
 
-
 void Proj4Projection::geos() {
     userEnveloppe_->clear();
     PCEnveloppe_->clear();
@@ -556,16 +595,15 @@ void Proj4Projection::geos() {
     max_pcx_ = -DBL_MAX;
     max_pcy_ = -DBL_MAX;
 
-
     map<double, vector<double> > helper;
 
     for (int lat = projection_->minlat_; lat <= projection_->maxlat_; lat++) {
         helper.insert(make_pair(lat, vector<double>()));
 
         for (int lon = projection_->minlon_; lon <= projection_->maxlon_; lon++) {
-            double x  = lon * DEG_TO_RAD;
-            double y  = DEG_TO_RAD * lat;
-            int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
 
             if (!error) {
                 helper[lat].push_back(lon);
@@ -573,15 +611,12 @@ void Proj4Projection::geos() {
         }
     }
 
-
     // now we create the envelope...
     for (map<double, vector<double> >::iterator lat = helper.begin(); lat != helper.end(); ++lat) {
         if (lat->second.empty())
             continue;
 
-
         add(lat->second.front(), lat->first);
-        // userEnveloppe_->push_back(PaperPoint(lat->second.front(), lat->first));
     }
     // now reverse!
 
@@ -590,7 +625,6 @@ void Proj4Projection::geos() {
             continue;
 
         add(lat->second.back(), lat->first);
-        // userEnveloppe_->push_back(PaperPoint(lat->second.back(), lat->first));
     }
 
     map<double, vector<double> >::iterator last = helper.begin();
@@ -601,17 +635,112 @@ void Proj4Projection::geos() {
 
     for (vector<double>::reverse_iterator lon = last->second.rbegin(); lon != last->second.rend(); ++lon) {
         add(*lon, last->first);
-        // userEnveloppe_->push_back(PaperPoint(*lon, last->first));
     }
 }
 
+magics::Polyline& Proj4Projection::getSimplePCBoundingBox() const {
+    static magics::Polyline box;
+    box.box(PaperPoint(min_pcx_, min_pcy_), PaperPoint(max_pcx_, max_pcy_));
+
+    return box;
+}
+
+void Proj4Projection::tpers() {
+    userEnveloppe_->clear();
+    PCEnveloppe_->clear();
+    // here we have to prepare the enveloppe!
+    min_pcx_ = DBL_MAX;
+    min_pcy_ = DBL_MAX;
+    max_pcx_ = -DBL_MAX;
+    max_pcy_ = -DBL_MAX;
+
+    map<double, vector<double> > helper;
+
+    // projection_view_latitude_
+    // projection_view_longitude_
+
+    double lastlon;
+    int missing = -99999;
+    double step = 0.1;
+    for (double lat = projection_view_latitude_; lat <= 90; lat += step) {
+        lastlon = missing;
+        for (double lon = projection_view_longitude_; lon <= projection_view_longitude_ + 360; lon += step) {
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
+            if (error) {
+                // we reach a border!
+
+                if (lastlon != missing) {
+                    add(lastlon, lat);
+                }
+                break;
+            }
+
+            lastlon = lon;
+        }
+    }
+    for (double lat = 90; lat >= projection_view_latitude_; lat -= step) {
+        lastlon = missing;
+        for (double lon = projection_view_longitude_; lon >= projection_view_longitude_ - 360; lon -= step) {
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
+            if (error) {
+                // we reach a border!
+                if (lastlon != missing) {
+                    add(lastlon, lat);
+                }
+                break;
+            }
+
+            lastlon = lon;
+        }
+    }
+
+    for (double lat = projection_view_latitude_; lat >= -90; lat -= step) {
+        lastlon = missing;
+        for (double lon = projection_view_longitude_; lon >= projection_view_longitude_ - 360; lon -= step) {
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
+            if (error) {
+                // we reach a border!
+                if (lastlon != missing) {
+                    add(lastlon, lat);
+                }
+                break;
+            }
+
+            lastlon = lon;
+        }
+    }
+    for (double lat = -90; lat <= projection_view_latitude_; lat += step) {
+        lastlon = missing;
+        for (double lon = projection_view_longitude_; lon <= projection_view_longitude_ + 360; lon += step) {
+            double x  = lon;
+            double y  = lat;
+            int error = helper_->convert(x, y);
+            if (error) {
+                // we reach a border!
+                if (lastlon != missing) {
+                    add(lastlon, lat);
+                }
+                break;
+            }
+
+            lastlon = lon;
+        }
+    }
+
+    userEnveloppe_->push_back(userEnveloppe_->front());
+    PCEnveloppe_->push_back(PCEnveloppe_->front());
+}
 
 void Proj4Projection::boundingBox(double& xmin, double& ymin, double& xmax, double& ymax) const {
-    if (!from_) {
-        from_ = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84");
-
+    if (!helper_) {
         projection_ = Epsg::find(definition_);
-        to_         = pj_init_plus(projection_->definition());
+        helper_     = new LatLonProjP(projection_->definition());
     }
 
     ymin = gridMinLat_;
@@ -668,37 +797,43 @@ double Proj4Projection::getMaxPCY() const {
     return max_pcy_;
 }
 
-
+void Proj4Projection::fill(double& width, double& height) {
+    Transformation::fill(width, height);
+    setting_       = "projection";
+    min_longitude_ = min_pcx_;
+    min_latitude_  = min_pcy_;
+    max_longitude_ = max_pcx_;
+    max_latitude_  = max_pcy_;
+}
 void Proj4Projection::gridLongitudes(const GridPlotting& grid) const {
-    magics::Polyline boundaries;
-
-    for (auto point = PCEnveloppe_->begin(); point != PCEnveloppe_->end(); ++point) {
-        boundaries.push_back(*point);
-    }
-
-    grid.add(boundaries);
-
-
     vector<double> longitudes = grid.longitudes();
 
-
     const double step = 0.5;
-    longitudes.push_back(180);
+    // longitudes.push_back(180);
     const vector<double>::const_iterator lon_end = longitudes.end();
-
+    // Always use full globe !
+    double min = -90;
+    double max = 90;
     for (vector<double>::const_iterator lon = longitudes.begin(); lon != lon_end; ++lon) {
         magics::Polyline poly;
         poly.setAntiAliasing(false);
 
-        for (double lat = gridMinLat_; (lat == gridMaxLat_ || lat < gridMaxLat_ + step); lat += step) {
-            PaperPoint p(*lon, lat);
-            if (userEnveloppe_->within(p))
-                poly.push_back((*this)(UserPoint(*lon, lat)));
-        }
+        for (double lat = min; lat < max; lat += step) {
+            PaperPoint p = (*this)(UserPoint(*lon, lat));
+            if (PCEnveloppe_->within(p)) {
+                poly.push_back(p);
+            }
+            else {
+                grid.add(poly);
 
+                poly = magics::Polyline();
+                poly.setAntiAliasing(false);
+            }
+        }
 
         grid.add(poly);
     }
+
     grid.addFrame(*PCEnveloppe_);
 }
 
@@ -708,16 +843,24 @@ void Proj4Projection::gridLatitudes(const GridPlotting& grid) const {
     const double step                            = 0.5;
     const vector<double>::const_iterator lat_end = latitudes.end();
 
+    // Always use full globe !
+    double min = -180;
+    double max = 360;
+
     for (vector<double>::const_iterator lat = latitudes.begin(); lat != lat_end; ++lat) {
         magics::Polyline poly;
         poly.setAntiAliasing(false);
-        for (double lon = gridMinLon_; lon <= gridMaxLon_ + step; lon += step) {
-            PaperPoint p(lon, *lat);
-            if (userEnveloppe_->within(p))
-                poly.push_back((*this)(UserPoint(lon, *lat)));
+        for (double lon = min; lon <= max; lon += step) {
+            PaperPoint p = (*this)(UserPoint(lon, *lat));
+            if (PCEnveloppe_->within(p))
+                poly.push_back(p);
+            else {
+                grid.add(poly);
+                poly = magics::Polyline();
+                poly.setAntiAliasing(false);
+            }
         }
 
-
         grid.add(poly);
     }
 
@@ -756,7 +899,6 @@ void Proj4Projection::labels(const LabelPlotting& label, DrawingVisitor& visitor
     }
 }
 
-
 void Proj4Projection::labels(const LabelPlotting& label, LeftAxisVisitor& visitor) const {
     if (projection_->method_ == "simple") {
         const vector<double>& latitudes = label.latitudes();
@@ -985,15 +1127,15 @@ void Proj4Projection::reprojectSpeedDirection(const PaperPoint& point, pair<doub
     double x = point.x_;
     double y = point.y_;
 
-    double u = x + (sin(wind.second * DEG_TO_RAD));
-    double v = y + (cos(wind.second * DEG_TO_RAD));
+    double u = x + (sin(RAD(wind.second)));
+    double v = y + (cos(RAD(wind.second)));
 
     fast_reproject(x, y);
     fast_reproject(u, v);
 
     double rotation = atan2((u - x), (v - y));
 
-    wind.second = (rotation * RAD_TO_DEG);
+    wind.second = DEG(rotation);
 }
 
 void Proj4Projection::revert(const vector<std::pair<double, double> >& in,
@@ -1006,23 +1148,23 @@ void Proj4Projection::revert(const vector<std::pair<double, double> >& in,
         PaperPoint p(x, y);
 
         if (PCEnveloppe_->within(p) == false) {
-            out.push_back(make_pair(-1000, -1000));
+            out.push_back(make_pair(HUGE_VAL, HUGE_VAL));
             continue;
         }
 
-        int error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
+        int error = helper_->revert(x, y);
 
         if (error) {
-            MagLog::error() << pj_strerrno(error) << " for " << pt->first << " " << pt->second << endl;
-            out.push_back(make_pair(-1000, -1000));
+            MagLog::error() << helper_->error(error) << " for " << pt->first << " " << pt->second << endl;
+            out.push_back(make_pair(HUGE_VAL, HUGE_VAL));
         }
         else {
-            double lon = x * RAD_TO_DEG;
+            double lon = x;
             if (lon > gridMaxLon_)
                 lon -= 360.;
             else if (lon < gridMinLon_)
                 lon += 360.;
-            double lat = y * RAD_TO_DEG;
+            double lat = y;
             out.push_back(make_pair(lon, lat));
         }
     }
@@ -1038,12 +1180,6 @@ void Proj4Projection::coastSetting(map<string, string>& setting, double abswidth
 
     double ratio = area / (abswidth * absheight);
 
-    // projFACTORS data = proj_factors(to_, xy);
-
-
-    // cout << "RATIO" << area << endl;
-
-
     std::string resol = "110m";
     if (ratio < 1000000)  // highest resolution
     {
@@ -1054,25 +1190,27 @@ void Proj4Projection::coastSetting(map<string, string>& setting, double abswidth
         resol = "50m";
     }
 
-    setting["resolution"]                = resol;
-    setting["land"]                      = resol + "/ne_" + resol + "_land";
-    setting["ocean"]                     = resol + "/ne_" + resol + "_ocean";
-    setting["coast"]                     = resol + "/ne_" + resol + "_coastline";
-    setting["rivers"]                    = resol + "/ne_" + resol + "_rivers_lake_centerlines";
-    setting["boundaries"]                = resol + "/ne_" + resol + "_admin_0_boundary_lines_land";
-    setting["administrative_boundaries"] = resol + "/ne_" + resol + "_admin_1_states_provinces";
+    setting["resolution"] = resol;
+    setting["land"]       = resol + "/ne_" + resol + "_land";
+    setting["ocean"]      = resol + "/ne_" + resol + "_ocean";
+    setting["coast"]      = resol + "/ne_" + resol + "_coastline";
+    setting["rivers"]     = resol + "/ne_" + resol + "_rivers_lake_centerlines";
+    setting["boundaries"] = resol + "/ne_" + resol + "_admin_0_boundary_lines_land";
+    //    setting["administrative_boundaries"] = resol + "/ne_" + resol +
+    //    "_admin_1_states_provinces";
 
-    MagLog::dev() << "GeoRectangularProjection::coastSetting[" << abswidth << ", " << absheight << "]->" << ratio
-                  << " resol: " << resol << endl;
+    //! \note Administraive borders hardcoded to 10m resolution (low res version
+    //! do not contain all info)
+    setting["administrative_boundaries"] = "10m/ne_10m_admin_1_states_provinces_lines";
 }
 
 void Proj4Projection::visit(MetaDataVisitor& visitor, double left, double top, double width, double height,
                             double iwidth, double iheight) {
     ostringstream java;
-    double w = getMaxPCX() - getMinPCX();
-    double h = getMaxPCY() - getMinPCY();
+    double w = max_pcx_ - min_pcx_;
+    double h = max_pcy_ - min_pcy_;
     java << "{";
-    java << "\"name\" : \"proj4\",";
+    java << "\"name\" : \"" << definition_ << "\",";
     java << "\"definition\" : \"" << definition_ << "\",";
     java << "\"proj4_definition\" : \"" << projection_->definition_ << "\",";
     java << "\"top\" : \"" << top << "\",";
@@ -1081,8 +1219,8 @@ void Proj4Projection::visit(MetaDataVisitor& visitor, double left, double top, d
     java << "\"height\" : \"" << height << "\",";
     java << "\"img_width\" : \"" << iwidth << "\",";
     java << "\"img_height\" : \"" << iheight << "\",";
-    java << "\"pcxmin\" : \"" << getMinPCX() << "\",";
-    java << "\"pcymin\" : \"" << getMinPCY() << "\",";
+    java << "\"pcxmin\" : \"" << min_pcx_ << "\",";
+    java << "\"pcymin\" : \"" << min_pcy_ << "\",";
     java << "\"pcwidth\" : \"" << w << "\",";
     java << "\"pcheight\" : \"" << h << "\"";
     java << "}";
@@ -1107,12 +1245,13 @@ bool Proj4Projection::fast_reproject(double& x, double& y) const {
         x -= (factor - width_);
     }
 
-    x *= DEG_TO_RAD;
-    y *= DEG_TO_RAD;
-    int error = pj_transform(from_, to_, 1, 1, &x, &y, NULL);
+    int error = helper_->convert(x, y);
 
     if (error) {
-        // MagLog::warning()  << pj_strerrno(error) << " for " << x << " " << y << endl;
+        x = HUGE_VAL;
+        y = HUGE_VAL;
+        // MagLog::warning()  << pj_strerrno(error) << " for " << x << " " << y <<
+        // endl;
         return false;
     }
     if (wraparound_)
@@ -1161,13 +1300,11 @@ void Proj4Projection::setDefinition(const string& json) {
     set(node);
 }
 
-
 void Proj4Automatic::setMinMaxX(double min, double max) {
     min_longitude_ = min;
     max_longitude_ = max;
 }
 
-
 void Proj4Automatic::setMinMaxY(double min, double max) {
     min_latitude_ = min;
     max_latitude_ = max;
@@ -1197,7 +1334,6 @@ void Proj4Automatic::setMinMaxY(double min, double max) {
     init_ = true;
     fill(width_, height_);
 
-
     // Now apply teh aspect ratio and reintialise!
 
     min_latitude_  = gridMinLat_;
@@ -1205,7 +1341,6 @@ void Proj4Automatic::setMinMaxY(double min, double max) {
     max_latitude_  = gridMaxLat_;
     max_longitude_ = gridMaxLon_;
 
-
     init();
 }
 
@@ -1221,19 +1356,18 @@ void Proj4Automatic::aspectRatio(double& width, double& height) {
     height_ = height;
 }
 
-
 void Proj4Automatic::setNewPCBox(double minx, double miny, double maxx, double maxy) {
-    double x  = minx;
-    double y  = miny;
-    int error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
+    gridMinLon_ = minx;
+    gridMinLat_ = miny;
+    int error   = helper_->revert(gridMinLon_, gridMinLat_);
+    if (error) {
+        MagLog::error() << "Error Proj4Automatic::setNewPCBox to check " << endl;
+    }
 
-    gridMinLon_ = x * RAD_TO_DEG;
-    gridMinLat_ = y * RAD_TO_DEG;
-
-    x     = maxx;
-    y     = maxy;
-    error = pj_transform(to_, from_, 1, 1, &x, &y, NULL);
-
-    gridMaxLon_ = x * RAD_TO_DEG;
-    gridMaxLat_ = y * RAD_TO_DEG;
+    gridMaxLon_ = maxx;
+    gridMaxLat_ = maxy;
+    error       = helper_->revert(gridMaxLon_, gridMaxLat_);
+    if (error) {
+        MagLog::error() << "Error Proj4Automatic::setNewPCBox to check " << endl;
+    }
 }
